/**
  * Classe abstraite reprsentant le calcul gnral de l'optimisation
  * de placement de labels.
  * On lui fournit un tableau de points  labeliser et pour chaque point
  * un rectangle de taille LxH reprsentant le texte  placer.
  */

#ifndef _LABELPLACEMENT_H_
#define _LABELPLACEMENT_H_

#include <qmutex.h>

//la structure de donnes reprsentant les labels  placer
#include "Map.h"

//la structure de donnes repsentant la solution du placement
#include "Solution.h"

class LabelPlacement
{
public :
      
	/**
	  * Les constructeurs.
	  */
	LabelPlacement();
	virtual ~LabelPlacement();
   
	/**
	  * Commence le calcul.
	  */
	virtual void compute()=0;

	/**
	  * Renvois le nombre initial de chevauchement.
	  */
	virtual int getNbInitialOverlap() const=0;

	/**
	  * Retourne le temps de calcul.
	  */
	int getComputationTime() const
	{
		return this->computationTime;
	}
   
	/**
	  * Renvois la solution trouve.
	  */
	virtual const Solution* getSolution() const;

	/**
	  * Enclenche le mode pas  pas.
	  */
	virtual void setStepByStepMod(bool b)
	{
		this->stepByStep = b;
	}

	/**
	  * Passe  l'itration suivante. Renvois true si il n'y en a plus.
	  */
	virtual bool nextStep()
	{
		if(this->stepByStepMutex.locked())
			this->stepByStepMutex.unlock();
		return false;
	}

protected :

	Solution* solution; //la solution

	int computationTime; //le temps de calcul

	bool stepByStep;
	QMutex stepByStepMutex;
};

#endif
