#include "Map.h"

#include <iostream>
using namespace std;

#include <qfontmetrics.h>

const double Map::marginPercent(0.05);

//###############//
MapElement::MapElement(Point<double> p, double l, double h)
{
	this->position = p;
	this->l = l;
	this->h = h;
	this->weight = 0; 
	this->text = "";
}

//###############//
MapElement::MapElement(Point<double> p, double l, double h, double w)
{
	this->position = p;
	this->l = l;
	this->h = h;
	this->weight = w; 
	this->text = "";
}

//###############//
MapElement::MapElement(Point<double> p, double w, string text, QFont font)
{
	this->position = p;
	this->weight = w; 
	this->text = text;
	this->font = font;
	QFontMetrics metrics(font);
	this->h = double(metrics.height());
	this->l = double(metrics.width(QString(this->text)));	
}

//###############//
MapElement::MapElement()
{
	this->position = Point<double>(0, 0);
	this->l = 0;
	this->h = 0;
	this->weight = 0;
}

//###############//
bool MapElement::haveText() const
{
	return this->text.size() != 0;
}

//###############//
int Map::getNbPointLabel() const
{
	return int(this->points->size());
}

//###############//
MapElement* Map::getPointLabel(int numLabel) const
{
	return (*this->points)[numLabel];
}

//###############//
Point<double> Map::getMinLimit() const
{
	return Point<double>(
		this->minLimit.x - Map::marginPercent * (this->maxLimit.x - this->minLimit.x),
		this->minLimit.y - this->marginPercent * (this->maxLimit.y - this->minLimit.y));
}

//###############//
Point<double> Map::getMaxLimit() const
{
	return Point<double>(
		this->maxLimit.x + Map::marginPercent * (this->maxLimit.x - this->minLimit.x),
		this->maxLimit.y + this->marginPercent * (this->maxLimit.y - this->minLimit.y));
}

//###############//
void Map::addPointLabel(MapElement* point)
{
	//effectue des tests pour dterminer la taille de la zone
	if(point->position.x + point->l > this->maxLimit.x)
		this->maxLimit.x = point->position.x + point->l;
	if(point->position.y + point->h > this->maxLimit.y)
		this->maxLimit.y = point->position.y + point->h;
	if(point->position.x - point->l < this->minLimit.x)
		this->minLimit.x = point->position.x - point->l;
	if(point->position.y - point->h < this->minLimit.y)
		this->minLimit.y = point->position.y - point->h;

	this->points->push_back(point);
}

#include <iostream>
using namespace std;

//###############//
void Map::doMasking(double* z)
{
	//parcours les points et enlve ceux qui ne sont pas dans la zone

	vector<MapElement*>* tmp = new vector<MapElement*>();

	for(vector<MapElement*>::iterator i = this->points->begin(); i != this->points->end(); i++)
		if((*i)->position.x > z[0] && (*i)->position.x < z[2] &&
			(*i)->position.y < z[1] && (*i)->position.y > z[3])
				tmp->push_back(*i);

	vector<MapElement*>* tmp2 = this->points;
	this->points = tmp;
	delete tmp2;
}

