/**
  * Classe reprsentant la solution aprs l'optimisation du placement.
  */

#ifndef _SOLUTION_H_
#define _SOLUTION_H_

#include <vector>
#include <iostream>
using namespace std;

#include "Point.cpp"

class Solution
{
public :

	/**
	  * Constructeur.
	  * p1 : le nombre de label 
	  */
	Solution(int);

	~Solution(); //destructeur
	
	/**
  	  * Renvois le placement relatif d'un label.
	  * p1 : Le numro du label
	  */
	Point<double> getLabelPlacement(int) const;

	/**
	  * Assigne l'emplacement d'un label.
	  * p1 : le numro du label
	  * p2 : l'emplacement
	  * p3 : entre-t-il en collision ?
	  */
	void setLabelPlacement(int, Point<double>, bool);

	/**
	  * Test si un label entre en collision.
	  * p1 : le numro du label
	  */
	bool labelIsOverlaping(int num) const
	{
		return (*this->overlaps)[num];
	}

	/**
	  * Dfinit le nombre de chevauchement
	  * p1 : le nombre de chevauchement.
	  */
	void setNumberOfOverlap(int nb)
	{
		this->nbOverlap = nb;
	}	

	/**
	  * Renvois le nombre de chevauchement.
	  */
	int getNumberOfOverlap() const
	{
		return this->nbOverlap;
	}

	/**
	  * Dfinit les labels qui sont cachs.
	  * p1 : les labels cachs
	  */
	void setHiddenLabels(const bool* hiddenLabels)
	{
		this->hiddenLabels = hiddenLabels;
	}

	/**
	  * Savoir si un label est masqu.
	  * p1 : le numro du label
	  */
	bool isHidden(int numLabel) const
	{
		if(this->hiddenLabels == 0) return false;

		return this->hiddenLabels[numLabel];
	}

	/**
	  * Savoir si le masquage est activ
	  */
	bool maskingOn() const
	{
		return this->hiddenLabels != 0;
	}

private :

	//le nombre de chevauchement
	int nbOverlap;
		
	//la position relative de chaque label
	Point<double>* solution;

	//les chevauchement des label
	vector<bool>* overlaps;

	//les labels cachs
	const bool* hiddenLabels;
};

#endif
