#include "DrawingZone.h"
using namespace Gui;

#include <iostream>
using namespace std;

#include <qpixmap.h>
#include <qfontmetrics.h>
#include <qrect.h>
#include <qpoint.h>

#include "../Point.h"

const QColor DrawingZone::backgroundColor(92, 201, 199);

//###############//
DrawingZone::DrawingZone(QWidget *parent, const char *name)
: QWidget(parent, name), map(map)
{
	this->map = 0;
	this->solution = 0;
	this->zoom = 1;
	this->gapX = 0;
	this->gapY = 0;
	this->setBackgroundMode(NoBackground);
}

//###############//
void DrawingZone::setMap(Map* map)
{
	this->map = map;
	this->solution = 0;
	this->repaint();
}

//###############//
void DrawingZone::setZoom(int v)
{
	this->zoom = double(v)/100;
	this->repaint();
}
int DrawingZone::getZoom() const
{
	return int(this->zoom * 100);
}

//###############//
void DrawingZone::setGapX(int gapX)
{
	this->gapX = double(gapX);
	this->repaint();
}

//###############//
void DrawingZone::setGapY(int gapY)
{
	this->gapY = double(gapY);
	this->repaint();
}

//###############//
void DrawingZone::setSolution(LabelPlacement* l)
{
	this->solution = l->getSolution();
	this->repaint();
}

//###############//
void DrawingZone::paintEvent(QPaintEvent*)
{

	QPixmap buffer(this->size());
	buffer.setOptimization(QPixmap::BestOptim);
	buffer.fill(DrawingZone::backgroundColor);
	QPainter paint;
	paint.begin(&buffer);

	//si il n'y a pas de map alors ne dessine rien
	if(this->map != 0)
	{

		//dcalage pour ramener la carte au point 0
		double decX =  this->map->getMinLimit().x;
		double decY =  this->map->getMinLimit().y;

		//hauteur de la zone, pour pouvoir inverser l'axe des y
		double zoneH = this->map->getMaxLimit().y - this->map->getMinLimit().y;
	
		//dcalage est zoom
		paint.translate(-this->gapX, -this->gapY);

		//cette solution fonctionne moyenne car elle ne prend pas en compte les double
		//paint.scale(this->zoom, this->zoom);

		//dssine les points et les labels
		for(int i=0; i<this->map->getNbPointLabel(); i++)
		{
			//ne dssine le label que si il est visible
			if(!(this->solution != 0 && this->solution->isHidden(i)))
			{
				paint.setPen(QColor(0, 0, 0));	

				//un bug de QT ??? : un rectangle dessin en ((10, 10), (20, 20))
				//ne supperpose pas un rectangle dessin en ((20, 20), (10, 10))
				//remde : mettre toujours la valeur infrieur en position 1... argh !
				int p1X = int((this->map->getPointLabel(i)->position.x - decX - 1.0)*
					this->zoom + 0.5);
				int p1Y = int((zoneH - (this->map->getPointLabel(i)->position.y - decY) - 1.0)*
					this->zoom + 0.5);
				int p2X = int((this->map->getPointLabel(i)->position.x - decX + 1.0)*
					this->zoom + 0.5);
				int p2Y = int((zoneH - (this->map->getPointLabel(i)->position.y - decY) + 1.0)*
					this->zoom + 0.5);
				paint.drawRect(QRect(
					QPoint(
						(p1X < p2X ? p1X : p2X),
						(p1Y < p2Y ? p1Y : p2Y)),
					QPoint(
						(p2X >= p1X ? p2X : p1X),
						(p2Y >= p1Y ? p2Y : p1Y))));

				//la position relative du label, par dfaut  (0,0)
				//si une solution est fournit alors prends la position de la solution
				Point<double> pos(0,0);
				bool overlap = false; //le label en chevauche un ou plusieurs autres ?
				if(this->solution != 0)
				{
					pos = this->solution->getLabelPlacement(i);	
					overlap = this->solution->labelIsOverlaping(i);		
				}


				p1X = int((this->map->getPointLabel(i)->position.x + pos.x - decX)*
					this->zoom + 0.5);
				p1Y = int((zoneH - (this->map->getPointLabel(i)->position.y + pos.y - decY))*
					this->zoom + 0.5);
				p2X = int((this->map->getPointLabel(i)->position.x +
					pos.x - decX + this->map->getPointLabel(i)->l)*this->zoom + 0.5);
				p2Y = int((zoneH - (this->map->getPointLabel(i)->position.y +
					pos.y - decY + this->map->getPointLabel(i)->h))*this->zoom + 0.5);
			
				if(!this->map->getPointLabel(i)->haveText())
				{

					//change la couleur du label en fonction si il entre en conflit ou non
					if(overlap) paint.setPen(QColor(255, 0, 0));
					else paint.setPen(QColor(0, 0, 255));	

					paint.drawRect(QRect(
						QPoint(
							(p1X < p2X ? p1X : p2X),
							(p1Y < p2Y ? p1Y : p2Y)),
						QPoint(
							(p2X >= p1X ? p2X : p1X),
							(p2Y >= p1Y ? p2Y : p1Y))));
				}
				else
				{

					paint.setPen(QColor(0, 0, 130));	

					QFont font = this->map->getPointLabel(i)->font;
					font.setPointSize(int(double(this->map->getPointLabel(i)->font.pointSize())*
						this->zoom+0.5));
					QFontMetrics metrics(font);
					paint.setFont(font);
	
					paint.drawText(
						QPoint(
							(p1X < p2X ? p1X : p2X),
							(p1Y < p2Y ? p1Y : p2Y) + metrics.ascent() ),
						QString(this->map->getPointLabel(i)->text)
					);	
				}
			}
		}
	}
	else //sinon affiche un texte invitant l'utilisateur a crer un problme
	{
		paint.setPen(QColor(0, 0, 0));	
		QFont font("Helvetica", 12);
		paint.setFont(font);
		QFontMetrics metrics(font);
		QString text("Right click to create an initial problem configuration");
		paint.drawText((this->width() - metrics.width(text))/2,
			(this->height() - metrics.height())/2 + metrics.ascent(), text);
	}
	paint.end();
	bitBlt(this, 0, 0, &buffer);
}
