/**
  * Reprsente une zone de dessin dans laquelle
  * des points ainsi que des labels peuvent tre dessins.
  */

#ifndef _DRAWING_ZONE_H_
#define _DRAWING_ZONE_H_

#include <qwidget.h>
#include <qpainter.h>
#include <qcolor.h>

#include "../Map.h"
#include "../LabelPlacement.h"
#include "../Solution.h"

namespace Gui
{
	class DrawingZone : public QWidget
	{
		Q_OBJECT
	public :
		/**
		  * Constructeur.
		  * p1-2 : voir la classe de base
		  */
		DrawingZone(QWidget *parent=0, const char *name=0);

		/**
		  * Dfinit la map.
		  * p1 : la map
		  */
		void setMap(Map*);

		/**
		  * Dfinit le zoom.
		  * p1 : la valeur du zoom
		  */
		void setZoom(int);

		/**
		  * Renvois le zoom courant.
		  */
		int getZoom() const;

	public slots :

		/**
		  * Dfinit le dcalage en X/Y.
		  * p1 : la valeur en X/Y
		  */
		void setGapX(int);
		void setGapY(int);

		/**
		  * Dfinit une nouvelle solution.
		  * p1 : le placement de label
		  */
		void setSolution(LabelPlacement*);

	protected :
		/**
		  * Appell lorsque la zone a besoin d'tre repeinte.
		  * p1 : l'venement
		  */
		void paintEvent(QPaintEvent *);

	private :	

		//la carte  dessiner
		Map* map;

		//la solution a dessiner
		const Solution* solution;

		//le zoom
		double zoom;

		//le dcalage
		double gapX, gapY;

		//la couleur du fond
		static const QColor backgroundColor; 
	};
}

#endif
