#include "MapVis.h"
using namespace Gui;

#include <iostream>
using namespace std;

#include <qlayout.h>
#include <qcursor.h>
#include <qfiledialog.h>
#include <qstring.h>

#include "../Point.h"
#include "../LabelPlacement.h"
#include "../MapFileReader.h"

//###############//
MapVis::MapVis(QWidget* parent, const char* name, WFlags f)
:QWidget(parent, name, f)
{
	this->map = 0;

	this->setCaption("Geographic label placement");
	
	//placement par grille
	QGridLayout* grid = new QGridLayout(this, 2, 2);

	this->drawingZone = new DrawingZone(this);
	this->ScBRight = new QScrollBar(Qt::Vertical, this);
	this->ScBBottom = new QScrollBar(Qt::Horizontal, this);

	grid->addWidget(this->drawingZone, 0, 0);
	grid->addWidget(this->ScBRight, 0, 1);
	grid->addWidget(this->ScBBottom, 1, 0);

	grid->setMargin(5);

	//dispose la fentre de visualisation
	this->resize(QSize(450, 400));
	this->move(QPoint(100, 100));

	//l'espace entre les fentres
	const int interWindowSpace = 50;

	//cration des autres fentres
	this->nav = new Nav(this);
	this->nav->move(QPoint(this->x() +
		this->frameGeometry().width() + interWindowSpace, this->y()));
	this->nav->show();
	this->solve = new Solve(this);
	this->solve->move(QPoint(this->x() + this->frameGeometry().width() +
		interWindowSpace, this->nav->y() + this->nav->frameGeometry().height() +
		interWindowSpace));
	this->solve->show();
	this->results = new Results(this);
	this->results->move(QPoint(this->x() + this->frameGeometry().width() +
		interWindowSpace,  this->solve->y() + this->solve->frameGeometry().height() +
		interWindowSpace));
	this->results->show();

	this->dlgRand = new RandGen(this, "RandGen");

	this->contextMenu = new QPopupMenu(this);
	Q_CHECK_PTR(this->contextMenu);
	this->contextMenu->insertItem("Random generator",  this, SLOT(randGen()));
	this->contextMenu->insertItem("Open a points file", this, SLOT(openPointsFile()));

	this->winSubmenu = new QPopupMenu(this);
	Q_CHECK_PTR(this->winSubmenu);	
	this->winSubmenu->setCheckable(true);

	this->navId = this->winSubmenu->insertItem("Nav", this, SLOT(winNav()));
	this->solveId = this->winSubmenu->insertItem("Solve", this, SLOT(winSolve()));
	this->resultsId = this->winSubmenu->insertItem("Results", this, SLOT(winResults()));

	this->contextMenu->insertSeparator();

	this->contextMenu->insertItem("Windows", winSubmenu);

	connect(this->nav, SIGNAL(zoomChanged(int)), this, SLOT(setZoom(int)));
	connect(this->solve, SIGNAL(solutionChanged(LabelPlacement*)),
		this->drawingZone, SLOT(setSolution(LabelPlacement*)));	
	connect(this->solve, SIGNAL(solutionChanged(LabelPlacement*)),
		this->results, SLOT(setResults(LabelPlacement*)));
	connect(this->solve, SIGNAL(computationInProgress()), this, SLOT(disableGui()));
	connect(this->solve, SIGNAL(computationFinished()), this, SLOT(enableGui()));

	connect(this->ScBRight, SIGNAL(valueChanged(int)), this->drawingZone, SLOT(setGapY(int)));
	connect(this->ScBBottom, SIGNAL(valueChanged(int)), this->drawingZone, SLOT(setGapX(int)));
}

//###############//
void MapVis::contextMenuEvent (QContextMenuEvent * )
{
	this->winSubmenu->setItemChecked(this->navId, this->nav->isVisible());
	this->winSubmenu->setItemChecked(this->solveId, this->solve->isVisible());
	this->winSubmenu->setItemChecked(this->resultsId, this->results->isVisible());
	this->contextMenu->exec( QCursor::pos() );
}

void MapVis::resizeEvent(QResizeEvent *)
{
	//on change les valeurs maximum des scrollbars
	this->changeScBMaxValue();
}

void MapVis::setZoom(int v)
{
	//change le zoome de la zone de dessin
	this->drawingZone->setZoom(v);

	//change les valeurs max des scrollbars :
	this->changeScBMaxValue();
}

//###############//
void MapVis::randGen()
{
	if(this->dlgRand->exec() == QDialog::Accepted)
	{
		
		Map* tmpMap = this->map;
		this->map = this->dlgRand->getMap();
		if(tmpMap != 0)	delete tmpMap;
		this->drawingZone->setMap(this->map);
		this->solve->setMap(this->map);
		this->changeScBMaxValue();
	};
}

#include <iostream>
using namespace std;

void MapVis::openPointsFile()
{
	QFileDialog* dlg = new QFileDialog(
		"./", "Map points file (*.txt)", this, 0, true);

	if(dlg->exec() == QDialog::Accepted)
	{
		Map* tmpMap = this->map;
		this->map = MapFileReader(dlg->selectedFile().latin1()).getMap();
		if(tmpMap != 0) delete tmpMap;
		this->drawingZone->setMap(this->map);
		this->solve->setMap(this->map);
		this->changeScBMaxValue();
	}
	delete dlg;
}

void MapVis::winNav()
{
	if(this->nav->isVisible())
		this->nav->hide();
	else
		this->nav->show();
}


void MapVis::winSolve()
{
	if(this->solve->isVisible())
		this->solve->hide();
	else
		this->solve->show();
}

void MapVis::winResults()
{
	if(this->results->isVisible())
		this->results->hide();
	else
		this->results->show();
}


void MapVis::disableGui()
{
	this->contextMenu->setDisabled(true);
}

void MapVis::enableGui()
{
	this->contextMenu->setDisabled(false);
}

void  MapVis::changeScBMaxValue()
{
	if(this->map != 0)
	{
		double l = this->map->getMaxLimit().x - this->map->getMinLimit().x;
		double h = this->map->getMaxLimit().y - this->map->getMinLimit().y;
		
		int maxValueX = int(h * (double(this->drawingZone->getZoom())/100.0)) -
			this->drawingZone->height();
		int maxValueY = int(l * (double(this->drawingZone->getZoom())/100.0)) -
			this->drawingZone->width();

		if(maxValueX <= 0) 
		{
			this->ScBRight->setEnabled(false);	
			this->ScBRight->setMaxValue(0);
		}
		else
		{
			this->ScBRight->setEnabled(true);
			this->ScBRight->setMaxValue(maxValueX);
		}

		if(maxValueY <= 0) 
		{
			this->ScBBottom->setEnabled(false);	
			this->ScBBottom->setMaxValue(0);
		}
		else
		{
			this->ScBBottom->setEnabled(true);
			this->ScBBottom->setMaxValue(maxValueY);
		}
	}
}
