/**
  * Permet de visualiser une map.
  */

#ifndef _MAP_VIS_H_
#define _MAP_VIS_H_

#include <qwidget.h>
#include <qnamespace.h> 
#include <qmenubar.h>
#include <qscrollbar.h>
#include <qpopupmenu.h>

#include "DrawingZone.h"
#include "Nav.h"
#include "Solve.h"
#include "Results.h"
#include "RandGen.h"

#include "../Map.h"
#include "../problemsGenerator/ProblemsGenerator.h"

namespace Gui
{
	class MapVis : public QWidget
	{
		Q_OBJECT
	public :
		/**
		  * Constructeur.
		  * p1-3 : voir classe de base
		  */
		MapVis(QWidget* parent=0, const char* name=0, WFlags f=0);

	public slots :
		void setZoom(int);

	private slots :
		/**
		  * Ouvre une boite de dialogue pour gnrer une carte.
		  */
		void randGen();	
		void openPointsFile();	
		void winNav();
		void winSolve();
		void winResults();

		/**
		  * Permet de dsactiver/activer l'interface pendant un calcul.
		  */
		void disableGui();
		void enableGui();

		/**
		  * Change les valeurs max des barres de scroll en fonction du zoom.
		  */
		void changeScBMaxValue();

	private : 
		/**
		  * Menu contextuel.
		  */
		void contextMenuEvent (QContextMenuEvent * );

		/**
		  * Redimensionnement de la fentre.
		  */
		void resizeEvent(QResizeEvent *);

		//la zone de dessin
		DrawingZone* drawingZone;

		//la fentre de navigation
		Nav* nav;

		//la fentre permettant d'entrer les paramtre des algos
		Solve* solve;

		//la fentre permettant d'entrer les paramtre des algos
		Results* results;

		RandGen* dlgRand;

		//la carte courante
		Map* map;

		//les scrollbars
		QScrollBar* ScBRight;
		QScrollBar* ScBBottom;
		
		//le menu contextuel
		QPopupMenu* contextMenu;
		QPopupMenu* winSubmenu;
		int navId, solveId, resultsId;
	};
}

#endif
