#include "RandGen.h"
using namespace Gui;

#include <qlabel.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qmessagebox.h>
#include <qvalidator.h>

#include "../problemsGenerator/ProblemsGenerator.h"

//###############//
RandGen::RandGen(QWidget* parent, const char* name)
: QDialog(parent, name, true)
{
	this->setCaption("Map generator");

	QLabel* lblNbLabel = new QLabel("Number of label", this);
	this->textNbLabel = new QLineEdit("200", this);
	this->textNbLabel->setValidator(new QIntValidator(10, 100000, this->textNbLabel));

	QLabel* lblSize = new QLabel("Zone size (x, y)", this);
	this->textSizeX = new QLineEdit("300", this);
	this->textSizeY = new QLineEdit("300", this);
	this->textSizeX->setValidator(new QDoubleValidator(1.0, 1e99, 2, this->textSizeX));
	this->textSizeY->setValidator(new QDoubleValidator(1.0, 1e99, 2, this->textSizeY));

	QLabel* lblSeed = new QLabel("Seed (-1 for time)", this);
	this->textSeed = new QLineEdit("-1", this);
	this->textSeed->setValidator(new QIntValidator(-1, 2147483647, this->textSeed));

	QLabel* lblLabelSize = new QLabel("Label size (x, y)", this);
	this->textLabelWidth = new QLineEdit("-1", this);
	this->textLabelHeight = new QLineEdit("-1", this);
	this->textLabelWidth->setValidator(
		new QDoubleValidator(-1.0, 1000, 2, this->textLabelWidth));
	this->textLabelHeight ->setValidator(
		new QDoubleValidator(-1.0, 1000, 2, this->textLabelHeight));

	QLabel* lblIntLabelPosition = new QLabel("Positions of labels are integer ?", this);
	this->checkIntLabelPosition = new QCheckBox(this);

	QPushButton* butCancel = new QPushButton("Cancel", this);
	QPushButton* butOk = new QPushButton("Ok", this);

	QGridLayout* grid = new QGridLayout(this, 5, 3);
	QGridLayout* butGrid = new QGridLayout(0, 1, 3);
	butGrid->addWidget(butCancel, 1,1);
	butGrid->addWidget(butOk, 1, 3);

	grid->addWidget(lblNbLabel, 1, 1);
	grid->addWidget(this->textNbLabel, 1, 3);

	grid->addWidget(lblSize, 2, 1);
	QGridLayout* gridSize = new QGridLayout(0, 1, 2);
	gridSize->addWidget(this->textSizeX, 1, 1);
	gridSize->addWidget(this->textSizeY, 1, 2);
	grid->addLayout(gridSize, 2, 3);

	grid->addWidget(lblSeed, 3, 1);
	grid->addWidget(this->textSeed, 3, 3);

	grid->addWidget(lblLabelSize, 4, 1);
	QGridLayout* gridLabelSize = new QGridLayout(0, 1, 2);
	gridLabelSize->addWidget(this->textLabelWidth, 1, 1);
	gridLabelSize->addWidget(this->textLabelHeight, 1, 2);
	grid->addLayout(gridLabelSize, 4, 3);

	grid->addWidget(lblIntLabelPosition, 5, 1);
	grid->addWidget(this->checkIntLabelPosition, 5, 3);

	grid->addMultiCellLayout(butGrid, 7, 7, 1, 3, Qt::AlignRight);

	grid->setColSpacing(2, 20);
	grid->setRowSpacing(6, 20);
	grid->setMargin(5);

	this->setMaximumSize(QSize(250, 0));
	this->resize(QSize(250, 0));

	butOk->setFocus();

	connect(butOk, SIGNAL(clicked()), this, SLOT(accept()));
	connect(butCancel, SIGNAL(clicked()), this, SLOT(reject()));
}

//###############//
Map* RandGen::getMap()
{
	return this->map;
}

//###############//
void RandGen::accept()
{
	//les limites des valeurs
	const int nbLabelLimit[2] = {10, 100000}; 
	const double sizeLimit[2] = {1.0, 1e99}; 
	const int seedLimit[2] = {-1, 2147483647}; 

	int nbLabel = this->textNbLabel->text().toInt();
	double* size = new double[2];
	size[0] = this->textSizeX->text().toDouble();
	size[1] = this->textSizeY->text().toDouble();
	int seed = this->textSeed->text().toInt();

	//controle la validit des valeurs
	if(nbLabel < nbLabelLimit[0] || nbLabel > nbLabelLimit[1])
		QMessageBox::information(this, "Randomize generator",
		QString("The number of label must set between ") + QString::number(nbLabelLimit[0]) +
		QString(" and ") + QString::number(nbLabelLimit[1]) + QString("."));
	else if(size[0] < sizeLimit[0] || size[0] > sizeLimit[1] ||
		size[1] < sizeLimit[0] || size[1] > sizeLimit[1])
		QMessageBox::information(this, "Randomize generator",
		QString("The size values must set between ") + QString::number(sizeLimit[0]) +
		QString(" and ") + QString::number(sizeLimit[1]) + QString("."));
	else if(seed < seedLimit[0] || seed > seedLimit[1])
		QMessageBox::information(this, "Randomize generator",
		QString("The seed value must set between ") + QString::number(seedLimit[0]) +
		QString(" and ") + QString::number(seedLimit[1]) + QString("."));
	else
	{
		double labelW = this->textLabelWidth->text().toDouble();
		double labelH = this->textLabelHeight->text().toDouble();
		double* labelWH = 0;
		if(labelW != -1&& labelH != -1)
		{
			labelWH = new double[2];
			labelWH[0] = labelW;
			labelWH[1] = labelH;
		}

		//gnre la map
		this->map = ProblemsGenerator(nbLabel, size, seed,
			labelWH, this->checkIntLabelPosition->isChecked()).getMap();
		QDialog::accept();
	}
}
