#include "Results.h"
using namespace Gui;

#include <qlayout.h>
#include <qlabel.h>
#include <qstring.h>

#include "../tabu/TabuLabelPlacement.h"

//###############//
Results::Results(QWidget* parent, const char* name)
: QDialog(parent, name, false, Qt::WStyle_Customize | Qt::WStyle_DialogBorder)
{
	this->setCaption("Results");

	QLabel* lblInitialOverlaping = new QLabel("Initial overlaping", this);
	this->textInitialOverlaping = new QLineEdit("", this);
	this->textInitialOverlaping->setReadOnly(true);

	QLabel* lblFinalOverlaping = new QLabel("Final overlaping", this);
	this->textFinalOverlaping = new QLineEdit("", this);
	this->textFinalOverlaping->setReadOnly(true);

	QLabel* lblNbIteration = new QLabel("Nb of iteration", this);
	this->textNbIteration = new QLineEdit("", this);
	this->textNbIteration->setReadOnly(true);

	QLabel* lblCompTime = new QLabel("Computation time [ms]", this);
	this->textCompTime = new QLineEdit("", this);
	this->textCompTime->setReadOnly(true);

	QLabel* lblInitTime = new QLabel("Initialisation  time [ms]", this);
	this->textInitTime = new QLineEdit("", this);
	this->textInitTime->setReadOnly(true);

	QGridLayout* grid = new QGridLayout(this, 5, 3);

	grid->addWidget(lblInitialOverlaping, 1, 1);
	grid->addWidget(this->textInitialOverlaping, 1, 3);

	grid->addWidget(lblFinalOverlaping, 2, 1);
	grid->addWidget(this->textFinalOverlaping, 2, 3);

	grid->addWidget(lblNbIteration, 3, 1);
	grid->addWidget(this->textNbIteration, 3, 3);

	grid->addWidget(lblInitTime, 4, 1);
	grid->addWidget(this->textInitTime, 4, 3);

	grid->addWidget(lblCompTime, 5, 1);
	grid->addWidget(this->textCompTime, 5, 3);

	grid->setColSpacing(2, 10);
	grid->setMargin(5);

	this->setMaximumSize(QSize(200, 0));
	this->resize(QSize(200, 0));
}	

//###############//
void Results::setResults(LabelPlacement* l)
{
	this->textInitialOverlaping->setText(QString::number(l->getNbInitialOverlap()));
	this->textFinalOverlaping->setText(QString::number(l->getSolution()->getNumberOfOverlap()));
	this->textNbIteration->setText(
		QString::number((dynamic_cast<TabuLabelPlacement*>(l))->getNbIteration()));
	this->textInitTime->setText(
		this->formatTime((dynamic_cast<TabuLabelPlacement*>(l))->getInitialisationTime()));
	this->textCompTime->setText(this->formatTime(l->getComputationTime()));
}

//###############//
QString Results::formatTime(int t) const
{
	return QString::number(t/3600000) + "H " +
		QString::number((t/60000)%60) + "Min " +
		QString::number((t/1000)%60) + "Sec";
}
