#include "Solve.h"
using namespace Gui;

#include <qlayout.h>
#include <qlabel.h>
#include <qvalidator.h>
#include <qmessagebox.h>
#include <qtimer.h>

#include "../tabu/TabuLabelPlacement.h"
#include "../tabu/PlacementDirectives.h"
//metaheuristique POPMUSIC
#include "../tabu/popmusic/PopTabuLabelPlacement.h"
using namespace Tabu;


Solve::Solve(QWidget* parent, const char* name)
: QDialog(parent, name, false, Qt::WStyle_Customize | Qt::WStyle_DialogBorder)
{
	this->map = 0;
	this->tabu = 0;
	this->timer = 0;

	//un timer pour rafraichir l'affichage en mode step by step
	this->timer = new QTimer(this);

	this->setCaption("Solve");

	QLabel* lblPopmusic = new QLabel("Heuristic", this);
	this->combBoxPopmusic = new QComboBox(false, this);
	this->combBoxPopmusic->insertItem( "Tabu Search" );
	this->combBoxPopmusic->insertItem( "POPMUSIC + TS v1" );
	this->combBoxPopmusic->insertItem( "POPMUSIC + TS v2" );	

	QLabel* lblSizeOfSubSet = new QLabel("Size of Subset", this);
	this->textSizeOfSubSet = new QLineEdit("70", this);
	this->textSizeOfSubSet->setValidator(new QIntValidator(5, 1000, this->textSizeOfSubSet));

	QLabel* lblPopSeed = new QLabel("Seed (-1 for time)", this);
	this->textPopSeed = new QLineEdit("-1", this);
	this->textPopSeed->setValidator(new QIntValidator(-1, 2147483647, this->textPopSeed));

	QLabel* lblNbIteration = new QLabel("Number of Iteration", this);
	this->textNbIteration = new QLineEdit("", this);
	this->textNbIteration->setValidator(new QIntValidator(1, 10000, this->textNbIteration));

	QLabel* lblRatio = new QLabel("Weight overlap over position [0..1]", this);
	this->textRatio = new QLineEdit("1", this);
	this->textRatio->setValidator(new QDoubleValidator(0.0, 1.0, 4, this->textRatio));

	QLabel* lblPointToLabelLength = new QLabel("Point to label length", this);
	this->textPointToLabelLength = new QLineEdit("0", this);
	this->textPointToLabelLength->setValidator(
		new QDoubleValidator(0.0, 100.0, 2, this->textPointToLabelLength));
	
	QLabel* lblInitialPlacementSeed =
		new QLabel("Initial label placement seed (-1 for time)", this);
	this->textInitialPlacementSeed = new QLineEdit("-1", this);
	this->textInitialPlacementSeed->setValidator(
		new QIntValidator(-1, 2147483647, this->textInitialPlacementSeed));

	QLabel* lblHide = new QLabel("Hide overlaping label ?", this);
	this->checkHide = new QCheckBox(this);

	QLabel* lblStepByStep = new QLabel("Step by step ?", this);
	this->checkStepByStep = new QCheckBox(this);

	QLabel* lblTimeStepByStep = new QLabel("Step [ms]", this);
	this->textTimeStepByeStep = new QLineEdit("300", this);
	this->textTimeStepByeStep->setValidator(
		new QIntValidator(0, 10000, this->textTimeStepByeStep));

	this->butSolve = new QPushButton("Solve", this);

	/**** PLACEMENT DES WIDGETS ****/
	QGridLayout* grid = new QGridLayout(this, 11, 4);

	grid->addMultiCellWidget(lblPopmusic, 1, 1, 1, 2);
	grid->addWidget(this->combBoxPopmusic, 1, 4);

	grid->addWidget(lblSizeOfSubSet, 2, 2);
	grid->addWidget(this->textSizeOfSubSet, 2, 4);

	grid->addWidget(lblPopSeed, 3, 2);
	grid->addWidget(this->textPopSeed, 3, 4);

	grid->addMultiCellWidget(lblNbIteration, 4, 4, 1, 2);
	grid->addWidget(this->textNbIteration, 4, 4);

	grid->addMultiCellWidget(lblRatio, 5, 5, 1, 2);
	grid->addWidget(this->textRatio, 5, 4);

	grid->addMultiCellWidget(lblPointToLabelLength, 6, 6, 1, 2);
	grid->addWidget(this->textPointToLabelLength, 6, 4);

	grid->addMultiCellWidget(lblInitialPlacementSeed, 7, 7, 1, 2);
	grid->addWidget(this->textInitialPlacementSeed, 7, 4);

	grid->addMultiCellWidget(lblHide, 8, 8, 1, 2);
	grid->addWidget(this->checkHide, 8, 4);

	grid->addMultiCellWidget(lblStepByStep, 9, 9, 1, 2);

	QBoxLayout* layTimeStepByStep = new QHBoxLayout();
	layTimeStepByStep->addWidget(this->checkStepByStep);
	layTimeStepByStep->addSpacing(10);
	layTimeStepByStep->addWidget(lblTimeStepByStep);
	layTimeStepByStep->addWidget(this->textTimeStepByeStep);

	grid->addLayout(layTimeStepByStep , 9, 4);

	grid->addMultiCellWidget(this->butSolve, 11, 11, 1, 4, Qt::AlignRight);

	grid->setColSpacing(1, 20);
	grid->setColSpacing(3, 10);
	grid->setRowSpacing(10, 20);
	grid->setMargin(5);

	this->setMaximumSize(QSize(150, 0));
	this->resize(QSize(150, 0));

	connect(butSolve, SIGNAL(clicked()), this, SLOT(solve()));
	connect(this->combBoxPopmusic, SIGNAL(activated(int)), this, SLOT(heuristicChanged(int)));
	connect(this->textTimeStepByeStep, SIGNAL(textChanged(const QString&)),
		this, SLOT(changeStepTime(const QString&)));

	this->heuristicChanged(0);
}

//###############//
void Solve::setMap(Map* map)
{
	this->map = map;
}

#include <iostream>
using namespace std;

//###############//
void Solve::run()
{
	emit computationInProgress();
	this->butSolve->setDisabled(true);

	this->tabu->setStepByStepMod(true);

	//ralise la calcule tabou
	(dynamic_cast<TabuLabelPlacement*>(this->tabu))->compute();

	//ralise le masquage si demand
	if(this->checkHide->isChecked())
		(dynamic_cast<TabuLabelPlacementMap*>(this->tabu))->hideLabels();	

	this->timer->stop();
	emit solutionChanged(this->tabu);

	this->butSolve->setDisabled(false);
	emit computationFinished();
}

//###############//
void Solve::oneStep()
{
	this->tabu->nextStep();
	emit solutionChanged(this->tabu);
}

//###############//
void Solve::heuristicChanged(int heuristic)
{
	if(heuristic == 0) //TS
	{
		this->textSizeOfSubSet->setEnabled(false); 
		this->textPopSeed->setEnabled(false); 
		this->textNbIteration->setText("1000");
	}
	else //POPMUSIC
	{
		this->textSizeOfSubSet->setEnabled(true); 
		this->textPopSeed->setEnabled(true); 
		this->textNbIteration->setText("70");
	}
}

//###############//
void Solve::changeStepTime(const QString& s)
{
	if(this->timer != 0 && this->timer->isActive())
		this->timer->changeInterval(s.toInt());
}

//###############//
void Solve::solve()
{
	//vrifie que la map existe
	if(this->map == 0)
	{
		QMessageBox::information(this, "Solve", "Any map of point has been defined.");
		return;
	}

	if(this->tabu != 0)
		delete this->tabu;

	int nbPosition = 4; //le nombre de position d'un label par rapport au point
	double preferences[] = { 0.0, 0.4, 0.6, 0.9 };

	//la taille de la liste de candidat par rapport  au nombre de label se chevauchant
	double candidateSizeBase = 0.73;
	double candidateReduceFactor = 1.3;
	double candidateGrowingFactor = 15;

	//la taille de la liste tabou par rapport  au nombre de label se chevauchant
	double tabuSize = 0.77;

	//aprs combien d'itration il faut recalculer la taille des listes (tabou et candidats)
	int nbIterationListSizeRecompute = 47;

	//taille minimale de la liste de candidat
	int minimalCandidateListSize = 18;

	//taille minimale de la liste tabou
	int minimalTabuListSize = 9;

	PlacementDirectives placementDirectives(preferences,
		nbPosition, this->textPointToLabelLength->text().toInt(),
		this->textRatio->text().toDouble());

	if(this->combBoxPopmusic->currentItem())
		this->tabu = new POPMUSIC::TabuLabelPlacement(this->map, placementDirectives,
			this->textSizeOfSubSet->text().toInt(), this->textPopSeed->text().toInt(),
			-1, this->combBoxPopmusic->currentItem(),
			this->textInitialPlacementSeed->text().toInt());
	else
		this->tabu = new TabuLabelPlacementMap(this->map, placementDirectives,
			this->textInitialPlacementSeed->text().toInt());
	
	//dfinit les paramtres de l'algorithme
	(dynamic_cast<TabuLabelPlacement*>(this->tabu))->setVariables(
			this->textNbIteration->text().toInt(), candidateSizeBase,
			candidateReduceFactor, candidateGrowingFactor,
			tabuSize, nbIterationListSizeRecompute, minimalCandidateListSize,
			minimalTabuListSize);

	if(this->checkStepByStep->isChecked())
	{
		this->start();
		connect(this->timer, SIGNAL(timeout()), this, SLOT(oneStep()));
		this->timer->start(this->textTimeStepByeStep->text().toInt());
	}
	else
	{
		this->tabu->setStepByStepMod(false);
		
		//ralise la calcule tabou
		(dynamic_cast<TabuLabelPlacement*>(this->tabu))->compute();

		//ralise le masquage si demand
		if(this->checkHide->isChecked())
			(dynamic_cast<TabuLabelPlacementMap*>(this->tabu))->hideLabels();	

		emit solutionChanged(this->tabu);
	}
}
