/**
  * Fentre permettant d'entrer les paramtres pour la rsolution de problme
  */

#ifndef _SOLVE_H_
#define _SOLVE_H_

#include <qwidget.h>
#include <qdialog.h>
#include <qlineedit.h>
#include <qcombobox.h>
#include <qstring.h>
#include <qcheckbox.h>
#include <qthread.h>
#include <qpushbutton.h>

#include "../Solution.h"
#include "../LabelPlacement.h"

namespace Gui
{
	class Solve : public QDialog, public QThread
	{
		Q_OBJECT

	public :
		/**
		  * Constructeur.
		  * p1-3 : voir classe de base
		  */
		Solve(QWidget* parent=0, const char* name=0);	

		/**
		  * Dfinit la map.
		  * p1 : la map
		  */
		void setMap(Map*);

		/**
		  * dmarre le thread pour un affichage en temps rel.
		  */
		virtual void run();

	signals :
		/**
		  * Est mit lorsque la solution change.
		  * p1 : le placement des labels.
		  */
		void solutionChanged(LabelPlacement*);

		/**
		  * Est mit pour signifier que le calcul a commenc.
		  */
		void computationInProgress();
		

		/**
		  * Est mit pour signifier que le calcul est fini.
		  */
		void computationFinished();
	
	private slots :

		/**
		  * Appell lorsque le bouton 'Solve' est press.
		  */
		void solve();

		/**
		  * Appell de manire priodique pour un affichage en temps rel.
		  */
		void oneStep();

		/**
		  * Appell lorsque l'utilisateur change d'heuristique (listbox).
		  */
		void heuristicChanged(int);

		/**
		  * Appell lorsque l'utilisateur change l'interval de rafraichissement.
		  */
		void changeStepTime(const QString&);

	private :
		
		//les widgets
    		QComboBox* combBoxPopmusic;
		QLineEdit* textSizeOfSubSet;
		QLineEdit* textPopSeed;

		QLineEdit* textNbIteration;
		QLineEdit* textRatio;
		QLineEdit* textPointToLabelLength;

		QLineEdit* textInitialPlacementSeed;

		QCheckBox* checkHide;
		QCheckBox* checkStepByStep;
		QLineEdit* textTimeStepByeStep;

		QPushButton* butSolve;

		QTimer* timer;

		//la map
		Map* map;

		//le placement de labels
		LabelPlacement* tabu;
	};
}

#endif
