/**
  * Travail de diplme 2003
  * Optimisation de placement de labels sur une carte gographique.
  * Fichier principal.
  *
  * Etudiant : Grgory Burri <greg.burri@net2000.ch>
  * Professeur : Eric Taillard
  * Date : 26.06.03
  * Compil avec gcc 3.3.1, le makefile est gnr
  * automatiquement  l'aide du programme 'qmake'
  * qui est fournit avec QT Designer.
  * QT designer ainsi que les bibliothques QT de developpement
  * sont fournit gratuitement  cette adresse :
  * http://www.trolltech.com/download/qt/x11.html
  *
  * Pour compiler taper simplement : qmake; make
  * 
  * L'indentation utilis et de type tabulaire,
  * une tabulation = 4 caractres espaces.
  * La longueur d'une ligne est de maximum 100 caractres pour
  * garantir une impression sans retour  la ligne intempestif, utilisez
  * l'outils 'check_length_of_line' pour vrifier la longueur des lignes.
  */

#include <iostream>
#include <string>
#include <ctime>
using namespace std;

//pour rsoudre un problme avec l'heuristique "Tabou"
#include "tabu/TabuLabelPlacement.h"
#include "tabu/PlacementDirectives.h"
//metaheuristique POPMUSIC
#include "tabu/popmusic/PopTabuLabelPlacement.h"
using namespace Tabu;

//pour gnrer un rsultat de type pixmap
#include "tools/ImageGenerator.h"
using namespace Tools;

//pour gnrer des donnes alatoires
#include "problemsGenerator/ProblemsGenerator.h"

//pour lire des fichiers de donnes
#include "MapFileReader.h"

//l'interface graphique
#include "gui/GuiBase.h"
using namespace Gui;

/**
  * Renvois un argument de la ligne de commande
  * si il n'existe pas alors lve une exception (pour viter un "segmentation fault")
  * p1 : le numro de l'argument  renvoyer
  * p2 : tous les arguments
  * p3 : le nombre d'argument total
  */
char* getArg(int numArg, char** argv, int argc)
{
	if(numArg >= argc) throw exception();
	return argv[numArg];
}

/**
  * Affiche l'utilisation du programme.
  * p1 : le nom de programme
  */
void printHelp(char* progName)
{
	cout << "Cartographics label placement with tabu search"
		" heuristic and POPMUSIC metaheuristic" << endl;
	cout << "Usage : " << progName << " [OPTIONS]..." << endl <<
	"Options are :" << endl <<
	"  -g <1 for enable GUI> " << endl <<
	"  [ -fo <file (type = potential overlap)> | -fp <file (type = liste of points)> ]" << endl <<
	"  -z <left> <top> <right> <bottom>" << endl << 
	"  -w <1 for enable hidding label with low weight and overlaping>"
		" (use only with -fp option)" << endl <<
	"  -p <1 for enable POPMUSIC euristic version 1, 2 for version 2>" << endl <<
	"  -u <size of subset (for POPMUSIC only)" << endl <<
	"  -ip <number of max iteration for POPMUSIC>" << endl <<
	"  -d <POPMUSIC seed (-1 for time)>" << endl <<
	"  -s <size X> <size Y>" << endl <<
	"  -i <number of iteration max>" << endl << 
	"  -ti <set how calcul the total compute time  "
		"1: initialisation time + algorithm time  "
		"0: algorithm time  "
		"2: initialisation time>" << endl <<
	"  -n <number of label>" << endl << 
	"  -r <weight overlap over position (ratio)>" << endl <<
	"  -v <initial candidate list factor size> <candidate list reduce factor> " <<
	"<candidate list growing factor> <tabu list factor size> " <<
	"<number of iteration before recompute size of lists> <minimal candidate list size>" <<
	"<minimal tabu list size>" << endl <<
	"  -e <random point placement seed (-1 for time)>" << endl <<
	"  -l <label to point length>" << endl <<
	"  -m <image size>" << endl <<
	"  -a <image filename>" << endl;
}

//###############//
int main(int argc, char** argv)
{
	/*~~~~~~~~~~~DONNES (avec valeurs par dfaut)~~~~~~~~~~~~*/
	double size[2] = {30, 30}; //la taille de la surface
	int nbLabel = 100; //le nombre de label  placer	
	/* statiques (pour l'instant) */	
		int nbPosition = 4; //le nombre de position d'un label par rapport au point
		double preferences[] = { 0.0, 0.4, 0.6, 0.9 };
	/* fin statique */
	int nbIteration = 1000; //le nombre d'itration
	double ratioOverlapPosition = 1; //l'importance du chevauchement par rapport  la position
	int seed = -1; //la germe pour la gnration alatoire
	double labelToPointLength = 0.0; //la distance entre le label et son point
	int imageSize = 1000; //taille de l'image rsultat
	string filename = "tmp/result"; //le nom du fichier de sortie

	//la taille de la liste de candidat par rapport  au nombre de label se chevauchant
	double candidateSizeBase = 0.73;
	double candidateReduceFactor = 1.3;
	double candidateGrowingFactor = 15;

	//la taille de la liste tabou par rapport  au nombre de label se chevauchant
	double tabuSize = 0.77;

	//aprs combien d'itration il faut recalculer la taille des listes (tabou et candidats)
	int nbIterationListSizeRecompute = 47;

	//taille minimale de la liste de candidat
	int minimalCandidateListSize = 18;

	//taille minimale de la liste tabou
	int minimalTabuListSize = 9;
	
	//si depuis un fichier : le nom du fichier
	char* dataFilename = 0;

	/* POPMUSIC */
	int sizeOfSubSet = 70; //le nombre de point contenu dans un sous ensemble
	int seedPOPMUSIC = -1;
	int nbIterationMaxPOP = -1; //le nombre d'itration maximumma
	/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

	//donnes depuis un fichier de donnes ? (si oui de quel type)
	enum FileType {NONE=0, OVERLAP, LIST_OF_POINT};
	FileType withDataFilename = NONE; 

	int withPOPMUSIC = 0; //utiliser la metaheuristique POPMUSIC
	bool withImageFile = false; //gnre les images png ?
	bool withHiddenLabel = false; //masque les labels de faible poids ?

	//doit-on restreindre le calcul sur un certaine zone ?
	bool widthMaskingZone = false;
	//la zone  masquer : 0=bord gauche, 1=bord haut, 2=bord droite, 3=bord bas
	double maskingZone[4];
	
	enum TimeType {COMPUTE=0, ALL, INITIALISATION};
	//par dfaut on ne compte que le temps d'initialisation
	TimeType howComputeTime = COMPUTE;

	//interface graphique ?
	bool withGui = false;

	try
	{
		//s'il n'y a pas d'argument alors affiche l'aide
		if(argc == 1)
		{
			printHelp(argv[0]);
			return 0;
		}

		//parcours les arguments fournis au programme
		for(int numArg=1; numArg < argc; numArg+=2)
		{
			string arg = string(argv[numArg]);
			if(arg == "--help" || arg == "help" || argc == 1)
			{	
				printHelp(argv[0]);
				return 0;
			}
			else
				if(arg == "-g")
				{
					withGui = string(getArg(numArg+1, argv, argc)) == "1";
				}
				else if(arg == "-f" || arg == "-fo")
				{
					dataFilename = getArg(numArg+1, argv, argc);
					withDataFilename = OVERLAP;
				}
				else if(arg == "-fp")
				{
					dataFilename = getArg(numArg+1, argv, argc);
					withDataFilename = LIST_OF_POINT;
				}
				else if(arg == "-v")
				{
					candidateSizeBase = atof(getArg(numArg+1, argv, argc));
					numArg++;
					candidateReduceFactor = atof(getArg(numArg+1, argv, argc));
					numArg++;
					candidateGrowingFactor  = atof(getArg(numArg+1, argv, argc));
					numArg++;
					tabuSize = atof(getArg(numArg+1, argv, argc));
					numArg++;
					nbIterationListSizeRecompute = atoi(getArg(numArg+1, argv, argc));
					numArg++;
					minimalCandidateListSize = atoi(getArg(numArg+1, argv, argc));
					numArg++;
					minimalTabuListSize = atoi(getArg(numArg+1, argv, argc));
				}
				else if(arg == "-w")
				{
					withHiddenLabel = string(getArg(numArg+1, argv, argc)) == "1";
				}
				else if(arg == "-p")
				{
					withPOPMUSIC = atoi(getArg(numArg+1, argv, argc));
				}
				else if(arg == "-u")
				{
					sizeOfSubSet  = atoi(getArg(numArg+1, argv, argc));
				}
				else if(arg == "-ip")
				{
					nbIterationMaxPOP = atoi(getArg(numArg+1, argv, argc));
				}
				else if(arg == "-d")
				{
					seedPOPMUSIC = atoi(getArg(numArg+1, argv, argc));
				}
				else if(arg == "-s")
				{
					size[0] = atof(getArg(numArg+1, argv, argc));
					numArg++;
					size[1] = atof(getArg(numArg+1, argv, argc));
				}
				else if(arg == "-i")
				{
					nbIteration = atoi(getArg(numArg+1, argv, argc));
				}
				else if(arg == "-ti")
				{
					switch(atoi(getArg(numArg+1, argv, argc)))
					{
						case 0 : howComputeTime = COMPUTE; break;
						case 1 : howComputeTime =  ALL; break;
						case 2 : howComputeTime =  INITIALISATION; break;	
					}					
				}
				else if(arg == "-n")
				{
					nbLabel = atoi(getArg(numArg+1, argv, argc));				
				}
				else if(arg == "-r")
				{
					ratioOverlapPosition = atof(getArg(numArg+1, argv, argc));	
				}
				else if(arg == "-z")
				{
					widthMaskingZone = true;
					maskingZone[0] = atof(getArg(numArg+1, argv, argc));
					numArg++;
					maskingZone[1] = atof(getArg(numArg+1, argv, argc));
					numArg++;
					maskingZone[2] = atof(getArg(numArg+1, argv, argc));
					numArg++;
					maskingZone[3] = atof(getArg(numArg+1, argv, argc));
				}
				else if(arg == "-e")
				{
					seed = atoi(getArg(numArg+1, argv, argc));
				}
				else if(arg == "-l")
				{
					labelToPointLength  = atof(getArg(numArg+1, argv, argc));
				}
				else if(arg == "-m")
				{
					imageSize = atoi(getArg(numArg+1, argv, argc));
				}
				else if(arg == "-a")
				{
					filename = string(getArg(numArg+1, argv, argc));
					withImageFile = true; 
				}
		}	
	}
	catch(exception)
	{
		cerr << "Arguments error" << endl;
		printHelp(argv[0]);
		exit(1);
	}

	if(withGui)
	{
		GuiBase gui;
		return gui.start(argc, argv);
	}

	//dfinit les prfrences de placement
	//(voir la classe "PlacementDirectives" pour plus de dtail)
	PlacementDirectives placementDirectives(preferences,
		nbPosition, labelToPointLength, ratioOverlapPosition);

	//la map sous forme de point
	Map* map = 0;

	if(withDataFilename == NONE)
		map = ProblemsGenerator(nbLabel, size, seed).getMap();
	else if(withDataFilename == LIST_OF_POINT)
		map = MapFileReader(dataFilename).getMap();

	//on enlve les point hors de la zone dfinit
	if(map != 0 && widthMaskingZone)
		map->doMasking(maskingZone);
	
	TabuLabelPlacement* tabu;

	//pour mesurer le temps de calcul de l'initialisation
	clock_t timeInit = clock();

	//donnes depuis un fichier ou les chevauchement potentiels sont dj indiqus
	if(withDataFilename == OVERLAP)
	{
		if(withPOPMUSIC)
			tabu = new POPMUSIC::TabuLabelPlacement(dataFilename, placementDirectives,
				sizeOfSubSet, seedPOPMUSIC, nbIterationMaxPOP, withPOPMUSIC);
		else
			tabu = new TabuLabelPlacementFile(dataFilename, placementDirectives);
	}
	else
	{	
		if(withPOPMUSIC)
			tabu = new POPMUSIC::TabuLabelPlacement(map, placementDirectives,
				sizeOfSubSet, seedPOPMUSIC, nbIterationMaxPOP, withPOPMUSIC);
		else
			tabu = new TabuLabelPlacementMap(map, placementDirectives);
	}

	//dfinit les paramtres de l'algorithme
	tabu->setVariables(nbIteration, candidateSizeBase,
			candidateReduceFactor, candidateGrowingFactor,
			tabuSize, nbIterationListSizeRecompute, minimalCandidateListSize,
			minimalTabuListSize);

	timeInit = (clock() - timeInit)/1000;

	if(withImageFile && (withDataFilename == NONE || withDataFilename == LIST_OF_POINT))
	{
		//gnre l'image de la solution initiale
		ImageGenerator pixmap_initial(map, tabu->getSolution(), imageSize);
		pixmap_initial.writeImageInAFile(filename + "_initial.png");
	}

	clock_t timeCompute = clock();

	//ralise la calcule tabou
	tabu->compute();

	//si on veux masquer les labels de poinds faible
	if(withHiddenLabel && withDataFilename == LIST_OF_POINT)
		dynamic_cast<Tabu::TabuLabelPlacementMap*>(tabu)->hideLabels();

	timeCompute = (clock() - timeCompute)/1000;

	clock_t time = clock();
	//en fonction du temp demand
	switch(howComputeTime)
	{
	case ALL :
		time = timeInit + timeCompute;
		break;

	case COMPUTE :
		time = timeCompute;
		break;

	case INITIALISATION :
		time = timeInit;
		break;
	}

	if(withImageFile && (withDataFilename == NONE || withDataFilename == LIST_OF_POINT))
	{
		//gnre l'image de la solution finale
		ImageGenerator pixmap_final(map, tabu->getSolution(), imageSize);
		pixmap_final.writeImageInAFile(filename + "_final.png");
	}

	cout << "The variables are : " << endl <<
	"size : " << size[0] << " x " << size[1] << endl <<
	"Number of label : " << tabu->getNumberPoint() << endl <<
	"Number of iteration max : " << nbIteration << endl <<
	"Weight overlap over position : " << ratioOverlapPosition << endl <<
	"Seed for the randomizer : " << seed << endl <<
	"Length label to point : " << labelToPointLength << endl <<
	"Number of iteration : " << tabu->getNbIteration() << endl;

	if(withPOPMUSIC)
		cout << "Size of the subset : " << sizeOfSubSet << endl;

	cout << endl << "Number of initial overlaping : " << tabu->getNbInitialOverlap() << endl <<
	"Number of overlaping : " << tabu->getSolution()->getNumberOfOverlap() << endl;
	//si le "masquage" des labels est enclench alors affiche le nombre de label visible
	if(withHiddenLabel)
	{
		int nb = 0;
		for(int i=0; i<tabu->getNumberPoint(); i++)
			if(!tabu->getSolution()->isHidden(i)) nb++;
		
		cout << "Number of unmasking label : " << nb << endl;
	}

	cout << "Time of computation : " << time << " ms" << endl <<
	"-------------------------" << endl;

	return 0;
}
