/**
  * Mmorise les cots des labels.
  * La liste est en permanance trie par cot ceci afin de pouvoir
  * facilement dfinir un ensemble de label ayant un cot suprieur.
  */

#ifndef _TABU_COSTLABELLIST_H_
#define _TABU_COSTLABELLIST_H_

#include <vector>
using namespace std;

#include "SolutionTabu.h"
#include "OverlapManager.h"

namespace Tabu
{
	/**
	  * Permet de stocker le cot des labels, lment de la liste
	  */
	struct CostLabel
	{
		/**
		  * Constructeurs.
		  * p1 : le cot
		  * p2 : le numro du label
		  * p3 : vrai si le label chevauche au moins un autre label
		  */
		CostLabel(double cost, int numLabel)
		: cost(cost), numLabel(numLabel){}

		CostLabel(){}

		double cost; //le cot de placement
		int numLabel; //le numro du label
	};

	class CostLabelList : public vector<CostLabel>
	{
	public :

		/**
		  * Constructeur.
		  * p1 : le placement des lments
		  * p2 : le gestionnaire des chevauchements
		  */
		CostLabelList(const SolutionTabu&, const OverlapManager&);

		/**
		  * Change le cot d'un label.
		  * p1 : un ponteur sur le label de la liste
		  * p2 : le nouveau cout
		  */
		void updateCost(CostLabelList::iterator, double);
	};
}

#endif

