#include "OverlapManager.h"
using namespace Tabu;

#include <algorithm>
#include <vector>
#include <iostream>
using namespace std;

#include <qdatetime.h> 

#include "../Point.cpp"

#define ns OverlapManager

//###############//
ns::OverlapManager(const Map* map, SolutionTabu* solution,
	const PlacementDirectives& pd)
: placementDirectives(pd), solution(solution)
{
	//pour mesurer le temps
	QTime time;
	time.start();

	double length = placementDirectives.getPointLabelLength();

	this->overlaps = new element*[map->getNbPointLabel()];

	//pour chaque label on dfinit 4 positions possibles
	for(int i=0; i<map->getNbPointLabel(); i++)
		this->overlaps[i] = new element[4];

	// chaque itration on calcul le coin infrieur gauche des 2 labels dont on veut
	//tester si ils se chevauchent
	Point<double> bottomLeft1;
	Point<double> bottomLeft2;

	//les hauteurs et largeurs des labels  tester
	double h1, h2, l1, l2;

	//pour chaque label
	for(int i=0; i<map->getNbPointLabel(); i++)
	{
		//pour chaque position du label
		for(int j=0; j<4; j++)
		{
			//pour chaque label2
			for(int i2=0; i2<map->getNbPointLabel(); i2++)
			{
				if(i==i2) continue;

				//pour chaque position du label2
				for(int j2=0; j2<4; j2++)
				{
					l1 = map->getPointLabel(i)->l;
					l2 = map->getPointLabel(i2)->l;
					h1 = map->getPointLabel(i)->h;
					h2 = map->getPointLabel(i2)->h;

					//calcul le coin infrieur gauche du premier label
					switch(j)
					{
					case 0 :
						bottomLeft1 = Point<double>(length, length); break;
					case 1 :
						bottomLeft1 = Point<double>(-length - l1, length); break;
					case 2 :
						bottomLeft1 = Point<double>(-length - l1, -length - h1); break;
					default :				
						bottomLeft1 = Point<double>(length, -length - h1); break;
					}
					bottomLeft1 += map->getPointLabel(i)->position;

					//calcul le coin infrieur gauche du deuxime label
					switch(j2)
					{
					case 0 :
						bottomLeft2 = Point<double>(length, length); break;
					case 1 :
						bottomLeft2 = Point<double>(-length - l2, length); break;
					case 2 :
						bottomLeft2 = Point<double>(-length - l2, -length - h2); break;
					default :				
						bottomLeft2 = Point<double>(length, -length - h2); break;
					}
					bottomLeft2 += map->getPointLabel(i2)->position;

					//test si le label 1 entre en conflit avec le label 2,
					//si oui alors l'indique dans la structure de donnes
					if(
						//si le point 1 se trouve dans le label 2 => chevauchement
						(map->getPointLabel(i)->position.x >= bottomLeft2.x &&
							map->getPointLabel(i)->position.x <= bottomLeft2.x + l2
						&&
						map->getPointLabel(i)->position.y >= bottomLeft2.y &&
							map->getPointLabel(i)->position.y <= bottomLeft2.y + h2)
						||
						//si le point 2 se trouve dans le label 1 => chevauchement
						(map->getPointLabel(i2)->position.x >= bottomLeft1.x &&
							map->getPointLabel(i2)->position.x <= bottomLeft1.x + l1
						&&
						map->getPointLabel(i2)->position.y >= bottomLeft1.y &&
							map->getPointLabel(i2)->position.y <= bottomLeft1.y + h1)
						||
						//si le label 1 chevauche le label 2 => chevauchement
						(((bottomLeft1.x >= bottomLeft2.x &&
							bottomLeft1.x <= bottomLeft2.x + l2) ||
					  	(bottomLeft1.x + l1 >= bottomLeft2.x &&
							bottomLeft1.x + l1 <= bottomLeft2.x + l2) ||
						(bottomLeft1.x <= bottomLeft2.x &&
							bottomLeft1.x + l1 >= bottomLeft2.x + l2))
						&&
						((bottomLeft1.y >= bottomLeft2.y &&
							bottomLeft1.y <= bottomLeft2.y + h2) ||
					  	(bottomLeft1.y + h1 >= bottomLeft2.y &&
							bottomLeft1.y + h1 <= bottomLeft2.y + h2) ||
						(bottomLeft1.y <= bottomLeft2.y &&	
							bottomLeft1.y + h1 >= bottomLeft2.y + h2)))
					)
						this->overlaps[i][j][i2].insert(this->overlaps[i][j][i2].end(), j2);
				}
			}
		}
	}

	this->initTime = time.elapsed();
}


//###############//
ns::OverlapManager(ifstream& file, SolutionTabu* solution,
	const PlacementDirectives& pd)
: placementDirectives(pd), solution(solution)
{
	this->overlaps = new element*[this->solution->size()];

	//pour chaque label on dfinit les positions possibles
	for(int i=0; i<int(this->solution->size()); i++)
		this->overlaps[i] = new element[this->placementDirectives.getNbPosition()];

	int nbOverlap; //le nombre de label que chevauche le label i

	//pour chaque point
	for(int i=0; i<int(this->solution->size()); i++)
	{
		//pour chaque position du label
		for(int j=0; j<this->placementDirectives.getNbPosition(); j++)
		{
			file >> nbOverlap;
			for(int k=0; k<nbOverlap; k++)
			{
				int tmp;
				file >> tmp;
				tmp--; //on commence  0 et non  1
				
				int numPoint = int(tmp / this->placementDirectives.getNbPosition());		
				int numPosition = tmp % this->placementDirectives.getNbPosition();
				
				//on ne tient pas compte du chevauchement avec soi mme
				if(numPoint == i) continue;

				this->overlaps[i][j][numPoint].insert(this->overlaps[i][j][numPoint].end(),
					numPosition);
			}
		}
	}
}

//###############//
ns::~OverlapManager()
{
	//pour chaque point
	for(int i=0; i<int(this->solution->size()); i++)
		delete[] this->overlaps[i];

	delete[] this->overlaps;
}

//###############//
int ns::overlap(int numLabel, bool number, int position = -1) const
{
	int nb = 0; //nombre de label en collision
	int pos = position;

	if(position == -1) pos = (*this->solution)[numLabel];

	//pour chaque potentiel label pouvant entrer en conflit
	for(element::iterator i = this->overlaps[numLabel][pos].begin();
		i!=this->overlaps[numLabel][pos].end(); i++)
		if(find(i->second.begin(), i->second.end(), (*this->solution)[i->first]) !=
			i->second.end())
		{
			nb++;
			if(!number) break;
		}

	return nb;
}

//###############//
int ns::numberOfLabelThatOverlap() const
{
	int nb = 0;
	
	for(int i = 0; i < int(this->solution->size()); i++)
		if(this->overlap(i, false) == 1) nb++;

	return nb;
}

//###############//
double ns::cost(int numLabel, int position) const
{
	int nbOverlap = this->overlap(numLabel, true, position);

	//alpha1 * nCh + alpha2 * position 
	return this->placementDirectives.getRatioOverlap() * nbOverlap +
		(1 - this->placementDirectives.getRatioOverlap()) *
		this->placementDirectives.getPreference((*this->solution)[numLabel]);
}

//###############//
double ns::cost(int numLabel) const
{
	return this->cost(numLabel, -1);
}

//###############//
//classe dfinissant un poids pour un label
struct Weight
{
	Weight(int num, double weight) : num(num), weight(weight){}
	int num;
	double weight;
};

//classe de comparaison entre 2 'Weight'
struct Comp
{
	int operator()(const Weight& w1, const Weight& w2)
	{
		return w1.weight > w2.weight;
	}
};

void ns::hideLabels(const Map* map, Solution* solution)
{
	//tableau permettant de savoir quel label est masqu
	bool* hiddenLabel = new bool[map->getNbPointLabel()];

	//un tableau de 'Weight', c'est  dire quel label a quel poids
	vector<Weight> weightSorted;

	//pour chaque label
	for(int i=0; i<map->getNbPointLabel(); i++)
	{
		//on introduit le poids et le numro du label dans le tableau
		weightSorted.push_back(Weight(i, map->getPointLabel(i)->weight));
		hiddenLabel[i] = false; //on en profite pour mettre  faux les labels cachs
	}

	//trie le tableau des poids de manire  les traiter dans l'ordre
	//en commencant par le plus lourd par la suite
	sort< vector<Weight>::iterator, Comp >(weightSorted.begin(), weightSorted.end(), Comp());

	//parcours chaque labels (du plus lourd au moins lourd)
	for(vector<Weight>::iterator i=weightSorted.begin(); i!=weightSorted.end(); i++)
	{
		//si il est cach alors se label ne peux plus en cacher un autre
		if(hiddenLabel[i->num]) continue;
		
		//pour chaque label avec lequel un conflit peut avoir lieu
		for(element::iterator j = this->overlaps[i->num][(*this->solution)[i->num]].begin();
			j!=this->overlaps[i->num][(*this->solution)[i->num]].end(); j++)
			
			//on regarde si la position de ce label existe dans l'ensemble
			//des positions dans lequelles il entre en conflit
			if(find(j->second.begin(), j->second.end(), (*this->solution)[j->first]) !=
				j->second.end())
					hiddenLabel[j->first] = true; //si il entre en conflit alors on le masque
			
	}

	//on donne  la solution le tableau des labels masqus
	solution->setHiddenLabels(hiddenLabel);
}

#undef ns
