/**
  * Gre les interactions entre les labels, on peut par exemple
  * demander combien de chevauchement cre un label.
  * Dfinit galement le cot de placement d'un label.
  */

#ifndef _TABU_OVERLAPMANAGER_H_
#define _TABU_OVERLAPMANAGER_H_

#include <map>
#include <set>
#include <vector>
#include <fstream>
#include <algorithm>
using namespace std;

#include "../Solution.h"
#include "../Map.h"

#include "SolutionTabu.h"
#include "PlacementDirectives.h"

#include "../Map.h"

namespace Tabu
{
	class OverlapManager
	{
	public :
		/**
		  * Constructeur.
		  * p1 : les label et leur point
		  * p2 : la solution
		  * p3 : les prfrences de placement
		  * TODO : rendre le nombre de position dynamique
		  * (il faut faire un gnrateur de placement autour du point).
		  */
		OverlapManager(const Map*, SolutionTabu*,const PlacementDirectives&);

		/**
		  * Constructeur.
		  * p1 : le fichier de donnes
		  * p2 : la solution
		  * p3 : les prfrences de placement
		  */
		OverlapManager(ifstream&, SolutionTabu*,const PlacementDirectives&);
		virtual ~OverlapManager();

		/**
		  * Renvois soit le nombre d'autre label que touche 'label' (p2 = true),
		  * ou soit si la label touche au moins un autre label (p2 = false).
		  * p1 : le numro du label
		  * p2 : ^
		  * p3 : la position du label, par dfaut prends sa position courante (facultatif)
		  */
		virtual int overlap(int, bool, int) const;

		/**
		  * Renvois le nombre de label en touchant au moins un autre.
		  */
		int numberOfLabelThatOverlap() const;

		/**
		  * Retourne le cot que le label n p1 engendre, calcul selon la formule :
		  * alpha1 * nb chevauchement + alpha2 * prfrence de placement.
		  * p1 : le numro du label
		  * p2 : la position du label (facultatif)
		  */
		double cost(int, int) const;
		double cost(int) const;

		/**
		  * Retourne le temps de l'initialisation
		  */
		int getInitTime() const
		{
			return this->initTime;
		}

		/**
		  * Dfinit les labels  masquer (en fonction de leur poids).
		  * p1 : les labels, contient les poids
	  	  * p2 : la solution dans laquelle le masquage va tre indiqu
		  */
		void hideLabels(const Map*, Solution*);

	protected :

		typedef map<int, set<int> > element;

		//l'ensemble des chevauchements possibles
		element** overlaps;

		//les directives de placement
		const PlacementDirectives& placementDirectives;

		//la solution, comment se positionne les labels,
		//rappel : il y a 4 positions posible : 0, 1, 2, 3
		SolutionTabu* solution;

		//le temps d'initialisation
		int initTime;
	};
}

#endif
