/**
  * Reprsente les prfrences de placement des
  * labels.
  * Un label peut-tre plac, relativement  son point, 
  * de quatre manires diffrentes :
  * - en haut  droite [0]
  * - en haut  gauche [1]
  * - en bas  gauche [2]
  * - en bas  droite [3]
  * Pour chaque position une prftence est donne,
  * elle part de 0 (meilleur placement) jusqu'a 1
  * (moins bon placement).
  */


#ifndef _TABU_PLACEMENTDIRECTIVES_H_
#define _TABU_PLACEMENTDIRECTIVES_H_

namespace Tabu
{
	class PlacementDirectives
	{
	public :

		/**
		  * Constructeur.
		  * p1: les prferences de chacune des position
		  * p2 : le nombre de position
		  * p3 : la distance entre les lablel et leur point
		  * p4 : l'importance du chevauchement par rapport  la position
		  *  entre 1 et 0, 1 tant le plus fort
		  */
		PlacementDirectives(const double*, int, double, double);
		PlacementDirectives(){}

		/**
		  * Renvois une prfrence.
		  * p1 : le numro prfrence
		  */
		double getPreference(int) const;

		/**
		  * Renvois le nombre de position.
		  */
		int getNbPosition() const;

		/**
		  * Renvois la distance entre le label et son point.
		  */
		double getPointLabelLength() const;

		/**
		  * Dfinit le nombre de position.
		  * p1 : le nombre de position
		  */
		void setNbPosition(int);

		/**
		  * Renvois le poids des chevauchements par rapport  la position.
		  */
		double getRatioOverlap() const;

	private :

		//les prfrences
		const double* positionPreferences;

		//le nombre de position du label
		int nbPosition;

		//la distance entre le label et son point
		double pointLabelLength;

		//l'importance du chevauchement par rapport  la position
		double ratioOverlap;
	};

	class ErrorDirectives {};
}

#endif
