#include "PopOverlapManager.h"
using namespace Tabu;

#include <list>
#include <map>
using namespace std;

#include "../../Point.cpp"

#define ns Tabu::POPMUSIC::OverlapManager

//###############//
ns::OverlapManager(const Map* map, Tabu::SolutionTabu* solution, const PlacementDirectives& pd)
: Tabu::OverlapManager::OverlapManager(map, solution, pd)
{}

//###############//
ns::OverlapManager(ifstream& file, Tabu::SolutionTabu* solution, const PlacementDirectives& pd)
: Tabu::OverlapManager::OverlapManager(file, solution, pd)
{}

//###############//
int ns::solutionValue() const
{
	int nb = 0;
	 //dsactive la vue du sous ensemble
	dynamic_cast<POPMUSIC::SolutionTabu*>(this->solution)->setSubSetActive(false);
	for(int i = 0; i < int(this->solution->size()); i++)
		if(Tabu::OverlapManager::overlap(i, false, -1) == 1) nb++;

	return nb;
}

//###############//
int ns::overlap(int numLabel, bool number, int position = -1) const
{
	bool subSetActive = dynamic_cast<POPMUSIC::SolutionTabu*>(this->solution)->subSetIsActive();

	 //dsactive la vue du sous ensemble
	dynamic_cast<POPMUSIC::SolutionTabu*>(this->solution)->setSubSetActive(false);
 
	int nb = Tabu::OverlapManager::overlap(subSetActive ?
		this->subSet[numLabel] : numLabel, number, position);

	//active la vue du sous ensemble
	dynamic_cast<POPMUSIC::SolutionTabu*>(this->solution)->setSubSetActive(subSetActive);

	return nb;
}

//###############//
void ns::setSubSetSize(int size)
{
	//if(this->subSetSize != 0) delete(this->subSet);,/m

	this->subSetSizeMax = size;
	this->subSet = new int[this->subSetSizeMax];
	this->subSetSize = new int;
	this->oldSolution = new int[this->solution->size()];

	static_cast<POPMUSIC::SolutionTabu*>
		(this->solution)->setSubSet(this->subSet, this->subSetSize);
}

//###############//
void ns::selectSubSet(int firstPoint)
{
	//permet de mmoriser les points qui ont dj t pris dans le sous ensemble
	bool pointTaken[this->solution->size()];
	for(int i = 0; i<int(this->solution->size()); i++)
		pointTaken[i] = false;
	pointTaken[firstPoint] = true;

	//compteurs
	int c1 = 1; //le nombre de point ajout dans le niveau courant
	*this->subSetSize = 1; //le nombre de point total du sous-ensemble

	//le premier point est initialement introduit dans le sous-ensemble
	this->subSet[0] = firstPoint;

	//pour savoir quand faut il arreter la construction du sous-ensemble
	bool goOut = false; 
	
	//parcours l'arbre des points en largeur
	while(!goOut)
	{
		//parcours les c1 derniers points ajouts  la liste
		int tmp = c1;
		c1 = 0;
		for(int i = *this->subSetSize-1; i > *this->subSetSize-tmp-1 && !goOut; i--)
		{
			//on parcours les positions du point courant
			for(int pos = 0; pos < this->placementDirectives.getNbPosition()  && !goOut; pos++)
			{
				//on parcours les points pouvant entrer en conflit
				for(element::const_iterator potentialConflictPoint =
					this->overlaps[this->subSet[i]][pos].begin();
					potentialConflictPoint != this->overlaps[this->subSet[i]][pos].end() && !goOut;
					potentialConflictPoint++)
				{
					//si le point n'a pas dj t prit alors on l'ajoute  la liste
					if(!pointTaken[potentialConflictPoint->first])
					{
						pointTaken[potentialConflictPoint->first] = true;
						this->subSet[*this->subSetSize] = potentialConflictPoint->first;
						c1++;
						(*this->subSetSize)++;
	
						//si la taille du sous-ensemble maximale
						// t atteinte alors arrete sort des boucles
						if(*this->subSetSize == this->subSetSizeMax) goOut = true;			
					}
				}
			}
		}
		//si aucun voisin n'a pu tre trouv alors arrte l'algorithme
		if(c1 == 0) goOut = true;
	}
}

//###############//
void ns::saveSolution()
{
	for(int i=0; i<*this->subSetSize; i++)
		this->oldSolution[this->subSet[i]] = (*this->solution)[i];
}

//###############//
void ns::rollbackToSavedSolution(set<int>& O, int version)
{
	for(int i=0; i<*this->subSetSize; i++)
	{
		(*this->solution)[this->subSet[i]] = this->oldSolution[this->subSet[i]];

		//si la version est gal  2 alors on insert tout le sous-ensemble dans O
		if(version == 2) O.insert(this->subSet[i]);
	}
}

//###############//
int ns::getSubSetSize() const
{
	return *this->subSetSize;
}

#undef ns
