/**
  * Cette classe redfinit lgrement la sementique de sa mre :
  *  Les fonctions 'numberOfLabelThatOverlap', 'overlap' et 'cost' 
  * s'appliquent  un sous ensemble des points.
  */

#ifndef _TABU_POPMUSIC_POPOVERLAPMANAGER_H_
#define _TABU_POPMUSIC_POPOVERLAPMANAGER_H_

#include <map>
#include <set>
#include <vector>
#include <fstream>
using namespace std;


#include "../../Map.h"
#include "../OverlapManager.h"
#include "PopSolutionTabu.h"

namespace Tabu
{
	namespace POPMUSIC
	{
		class OverlapManager : public Tabu::OverlapManager
		{
		public :
			/**
		  	  * Constructeur #1
			  * p1 : les label et leur point
		  	  * p2 : la solution
		  	  * p3 : les prfrences de placement
		  	  */
			OverlapManager(const Map*, Tabu::SolutionTabu*, const PlacementDirectives&);

			/**
		  	  * Constructeur #2
			  * p1 : le fichier de donnes
		  	  * p2 : la solution
		  	  * p3 : les prfrences de placement
		  	  */
			OverlapManager(ifstream&, Tabu::SolutionTabu*,const PlacementDirectives&);

			virtual ~OverlapManager()
			{
				delete[] this->subSet;
				delete this->subSetSize;
				delete[] this->oldSolution;
			}

			/**
			  * Renvois la valeur de la solution (le nombre de chevauchement)
			  */
			int solutionValue() const;

			/**
			  * Renvois soit le nombre d'autre label que touche 'label' (p2 = true),
			  * ou soit si la label touche au moins un autre label (p2 = false).
			  * p1 : le numro du label
			  * p2 : ^
			  * p3 : la position du label, par dfaut prends sa position courante (facultatif)
			  */
			int overlap(int, bool, int) const;

			/**
			  * Dfinit la taille (maximale) des sous-ensembles.
			  * p1 : la taille
			  */
			void setSubSetSize(int);

			/**
			  * Dfinit les lments du sous-ensemble en fonction d'un point de dpart.
			  * p1 : le point de dpart
			  */
			void selectSubSet(int);

			/**
			  * Mmorise la solution dfinit par le sous ensemble courant.
			  */
			void saveSolution();

			/**
			  * Restore la solution courante par l'ancienne
			  * (dans le cas ou la nouvelle et moins bonne)
			  * p1 : l'ensemble O
			  * p2 : la version de popmusic  appliquer
			  */
			void rollbackToSavedSolution(set<int>&, int);

			/**
			  * Renvois la taille du sous-ensemble courant.
			  */
			int getSubSetSize() const;

		private :

			//le sous ensemble
			int* subSet;

			//la taille maximale des sous-ensembles
			int subSetSizeMax;

			//la taille du sous-ensemble courant
			int* subSetSize;

			//pour stocker temporairement la solution (afin de pouvoir revenir en arrire)
			int* oldSolution;
		};
	}
}

#endif
