/**
  * Reprsente la solution mais permet de restreindre la vue  un sous ensemble
  * Ceci permet d'appliquer Tabou seulement  un sous-ensemble de point ce qui est
  * ncessaire dans le cas de POPMUSIC.
  */

#ifndef _POP_TABU_SOLUTIONTABU_H_
#define _POP_TABU_SOLUTIONTABU_H_

#include "../SolutionTabu.h"

namespace Tabu
{
	namespace	POPMUSIC
	{

		class SolutionTabu : public Tabu::SolutionTabu
		{	
		public :

			/**
			  * Constructeur, par dfaut la solution est de taille nulle.
			  */
			SolutionTabu() : Tabu::SolutionTabu(0)
			{
				this->subSetActive = false;
			}

			/**
			  * Constructeur.
			  * p1 : la taille de la solution
			  */
			SolutionTabu(int n) : Tabu::SolutionTabu(n)
			{
				this->subSetActive = false;
			}

			/**
			  * Permet d'acceder  la position d'un label.
			  * p1 : le numro du du label
			  */
			value_type& operator[](size_type i)
			{				
				return this->subSetActive ? this->at(subSet[i]) : this->at(i);
			}

			/**
			  * Permet d'acceder  la position d'un label, renvois un const.
			  * p1 : le numro du du label
			  */
			const value_type& operator[](size_type i) const
			{				
				return this->subSetActive ? this->at(subSet[i]) : this->at(i);
			}
		
			/**
			  * Active ou dsactive le sous ensemble de point,
			  * tout les autres points ne peuvent pas tre acceds.
			  * p1 : activation / dsactivation
			  */
			void setSubSetActive(bool active)
			{
				this->subSetActive = active;
			}
		
			/**
			  * Savoir si le sous-ensemble est actif.
			  */
			bool subSetIsActive() const
			{
				return this->subSetActive;
			}

			/**
			  * Renvois la taille de la solution,
			  * renvois la taille du sous-ensemble si il est actif.
			  */
			size_type size() const
			{
				if(this->subSetActive)
					return *this->subSetSize;
				else
					return Tabu::SolutionTabu::size();
			}

			/**
			  * Dfinit le sous ensemble.
			  * p1 : le sous-ensemble sous la forme d'un tableau de numro de point
			  * (ceux qui appartiennent au sous-ensemble).
			  * p2 : la taille du sous-ensemble
			  */
			void setSubSet(int* set, int* size)
			{
				this->subSet = set;
				this->subSetSize = size;
				this->subSetActive = true;
			}

		private :

			//la taille du sous ensemble
			int* subSetSize;

			//le sous ensemble
			int* subSet;

			//est-ce que la vue est limit au sous ensemble courant ?
			bool subSetActive;	
		};
	}
}

#endif
