#include "PopTabuLabelPlacement.h"
using namespace Tabu;

#include <cstdlib>
#include <fstream>
using namespace std;

#include <qdatetime.h> 

#define ns Tabu::POPMUSIC::TabuLabelPlacement

//###############//
ns::TabuLabelPlacement(char* name, PlacementDirectives pd,
	int subSetSize, int seed, int nbIterationMax, int version, int initialPlacementSeed)
: TabuLabelPlacementMap(), TabuLabelPlacementFile()
{
	srand(seed == -1 ? time(0) : seed);

	this->version = version;

	this->nbIterationMax = nbIterationMax;

	this->placementDirectives = pd;

	ifstream file(name);
	if(!file)
		this->fileError(name);

	int nbLabel, nbPosition;

	file >> nbLabel;
	file >> nbPosition;

	//solution qui ne servira  rien puisque aucune coordonn
	//n'est fournit dans les fichiers de donnes
	this->solution = new Solution(nbLabel);

	//la futur solution, par dfaut on les places  la meilleure position
	this->solutionTabu = new POPMUSIC::SolutionTabu(nbLabel);

	//cre la solution initiale
	this->creatInitialSolution(nbLabel,
		this->placementDirectives.getRatioOverlap(), initialPlacementSeed);

	this->placementDirectives.setNbPosition(nbPosition);

	//le gestionnaire de chevauchement
	this->om = new POPMUSIC::OverlapManager(file, this->solutionTabu,
		this->placementDirectives);

	this->initialisationTime = this->om->getInitTime();

	//convertit la solution tabu en solution gnrique
	Tabu::TabuLabelPlacementFile::convSolution();

	this->subSetSize = subSetSize;
}

//###############//
ns::TabuLabelPlacement(const Map* map, PlacementDirectives pd,
	int subSetSize, int seed, int nbIterationMax, int version, int initialPlacementSeed)
: TabuLabelPlacementMap(), TabuLabelPlacementFile()
{
	srand(seed == -1 ? time(0) : seed);

	this->version = version;

	this->nbIterationMax = nbIterationMax;

	this->placementDirectives = pd;
	this->map = map;	
	this->solution = new Solution(this->map->getNbPointLabel());

	//la futur solution, par dfaut on les places  la meilleure position
	this->solutionTabu = new POPMUSIC::SolutionTabu(this->map->getNbPointLabel());

	//cre la solution initiale
	this->creatInitialSolution(this->map->getNbPointLabel(),
		this->placementDirectives.getRatioOverlap(), initialPlacementSeed);

	//le gestionnaire de chevauchement
	this->om = new POPMUSIC::OverlapManager(this->map, this->solutionTabu,
		this->placementDirectives);

	this->initialisationTime = this->om->getInitTime();

	//convertit la solution tabu en solution gnrique (pas joli joli comme mthode)
	Tabu::TabuLabelPlacementMap::convSolution();

	this->subSetSize = subSetSize;
}

//###############//
void ns::compute()
{	
	//pour mesurer le temps
	QTime time;
	time.start();

	//mute l'overlap manager
	POPMUSIC::OverlapManager* omTmp = dynamic_cast<POPMUSIC::OverlapManager*>(this->om);

	//dfinit la taille maximale des sous-ensemble
	omTmp->setSubSetSize(this->subSetSize);

	//l'ensemble des points dont on ne peut plus amliorer le sous-problme
	set<int> O;

 	//dsactive la vue du sous ensemble
	dynamic_cast<POPMUSIC::SolutionTabu*>(this->solutionTabu)->setSubSetActive(false);

	int solutionValueAvant; //la valeur de la solution actuelle
	int solutionValueApres = omTmp->solutionValue();
	int solutionValueDebut = solutionValueApres; //mmorise le nombre de chevauchement initial


	//compte le nombre d'itration
	int nbIterationPOP = 0;

	//tant que tout les points ne se trouve pas dans O
	while(O.size() != this->solutionTabu->size() &&
		(this->nbIterationMax == -1 || nbIterationPOP < this->nbIterationMax))
	{
		nbIterationPOP++;

		solutionValueAvant = solutionValueApres;

		//on tire un point alatoirement qui ne se trouve pas dans O
		int point;
		do
 		{
			point = int((double(rand())/RAND_MAX)*this->solutionTabu->size());
		}while(O.find(point)!=O.end());

		//dfinit le sous ensemble
		omTmp->selectSubSet(point);

		//active la vue du sous ensemble
		dynamic_cast<POPMUSIC::SolutionTabu*>(this->solutionTabu)->setSubSetActive(true);

		//mmorise la solution avant l'optimisation
		omTmp->saveSolution();

		//on dsactive le mode pas  pas pour l'application de tabou
		bool stepByStepTmp = this->stepByStep;
		this->stepByStep = false;

		//applique l'optimisation
		Tabu::TabuLabelPlacement::compute();

		this->stepByStep = stepByStepTmp;

		//dsactive la vue du sous ensemble
		dynamic_cast<POPMUSIC::SolutionTabu*>(this->solutionTabu)->setSubSetActive(false); 

		//mode tape par tape
		if(this->stepByStep)
		{
			QTime time2;
			time2.start();
			this->solution->setNumberOfOverlap(solutionValueApres);
			//POPMUSIC::TabuLabelPlacement::convSolution();
			//convertit la solution tabu en solution gnrique (pas joli joli comme mthode) 
			if(this->map)
				Tabu::TabuLabelPlacementMap::convSolution();
			else
				Tabu::TabuLabelPlacementFile::convSolution();
			this->computationTime = time.elapsed();	
			this->nbIteration = nbIterationPOP;
			this->stepByStepMutex.lock();
			time = time.addMSecs(time2.elapsed());
		}

		solutionValueApres = omTmp->solutionValue();
		if(solutionValueApres < solutionValueAvant)
		{
			O.clear();
		}
		else
		{
			//revient  l'ancienne solution
			omTmp->rollbackToSavedSolution(O, this->version);
			
			//si c'est la version original alors on insert uniquement le point,
			//sinon on insert tout le sous problme
			if(this->version == 1) O.insert(point);

			solutionValueApres = solutionValueAvant;
		}
	}
		
	//dfinit les variables de rsultats
	this->nbInitialOverlap = solutionValueDebut;
	this->solution->setNumberOfOverlap(solutionValueApres);
	this->nbIteration = nbIterationPOP;
	this->computationTime = time.elapsed();
	//POPMUSIC::TabuLabelPlacement::convSolution();
	//convertit la solution tabu en solution gnrique (pas joli joli comme mthode) 
	if(this->map)
		Tabu::TabuLabelPlacementMap::convSolution();
	else
		Tabu::TabuLabelPlacementFile::convSolution();
}

//###############//
void ns::convSolution()
{

}
