/**
  * Classe implementant la placement des labels  l'aide
  * de la metaheuristique 'POPMUSIC'.
  * Cette classe se base sur la recherche tabu.
  */

#ifndef _POPTABULABELPLACEMENT_H_
#define _POPTABULABELPLACEMENT_H_

#include "../TabuLabelPlacement.h"
#include "PopOverlapManager.h"
#include "PopSolutionTabu.h"

namespace Tabu
{
	namespace POPMUSIC
	{
		class TabuLabelPlacement : public TabuLabelPlacementMap, public TabuLabelPlacementFile
		{
		public :
			/**
			  * Constructeur  partir d'un fichier.
		  	  * p1 : le nom du fichier
			  * p2 : les prfrences de placement
			  * p3 : la taille du sous ensemble
			  * p4 : la graine qui va dterminer l'ordre de slction des points
			  * p5 : le nombre maximum d'iteration (si -1 alors va jusqu'au bout)
			  * p6 : la version de popmusic : 1=version original 2=version modifi
			  * p7 : le germe de gnration de la solution initiale
			  */
			TabuLabelPlacement(char*, PlacementDirectives,
				int, int, int, int, int initialPlacementSeed=-1);

			/**
			  * p1 : les label et leur point
			  * p2 : les prfrences de placement
			  * p3 : la taille du sous ensemble
			  * p4 : la graine qui va dterminer l'ordre de slction des points
			  * p5 : le nombre maximum d'iteration (si -1 alors va jusqu'au bout)
			  * p6 : la version de popmusic : 1=version original 2=version modifi
			  * p7 : le germe de gnration de la solution initiale
			  */
			TabuLabelPlacement(const Map*, PlacementDirectives,
				int, int, int, int, int initialPlacementSeed=-1);

			/**
			  * Commence le calcul.
			  */
			void compute();

		protected :

			/**
			  * Convertit une "solution tabou" en une solution gnrique.
			  */
			void convSolution();

			//la taille des sous ensemble
			int subSetSize;

			//le nombre maximal de sous-ensemble
			int nbIterationMax;

			//la vesrion de popmusic
			int version;
		};
	}
}


#endif
