#include "ImageGenerator.h"
using namespace Tools;

#include <iostream>
using namespace std;

#include <qcolor.h>
#include <qstring.h>

#include "../Point.cpp"

#define ns ImageGenerator

//###############//  
ns::ImageGenerator(const Map* map, const Solution* solution, int bitmapSize)
:map(map), solution(solution)
{
	this->bitmapSize = bitmapSize;
	this->nbPixelBorder = 2;

	//dfinit la taille de la zone de placement
	Point<double> pMax(-99999, -99999);
	Point<double> pMin(99999, 99999);
	
	//parcours tous les points afin de cherche les extremes pour dfiinir la taille de la zone
	for(int i=0; i<this->map->getNbPointLabel(); i++)
	{
		double xMax = this->map->getPointLabel(i)->position.x + 
			(this->map->getPointLabel(i)->l + this->solution->getLabelPlacement(i).x > 0 ?
			this->map->getPointLabel(i)->l + this->solution->getLabelPlacement(i).x : 0);
		if(xMax > pMax.x) pMax.x = xMax;

		double yMax = this->map->getPointLabel(i)->position.y + 
			(this->map->getPointLabel(i)->h + this->solution->getLabelPlacement(i).y > 0 ?
			this->map->getPointLabel(i)->h + this->solution->getLabelPlacement(i).y : 0);
		if(yMax > pMax.y) pMax.y = yMax;

		double xMin =  this->map->getPointLabel(i)->position.x + 
			(this->solution->getLabelPlacement(i).x < 0 ?
			this->solution->getLabelPlacement(i).x : 0);
		if(xMin < pMin.x) pMin.x = xMin;

		double yMin =  this->map->getPointLabel(i)->position.y + 
			(this->solution->getLabelPlacement(i).y < 0 ?
			this->solution->getLabelPlacement(i).y : 0);
		if(yMin < pMin.y) pMin.y = yMin;
	}

	this->minCorner = pMin;

	//hauteur et largeur de la surface de placement
	double width = (pMax.x - pMin.x);
	double height = (pMax.y - pMin.y);

	//calcul le taille de l'image finale en fonction de la taille demand
	this->imageW = int(width > height ? this->bitmapSize - (2*this->nbPixelBorder) :
		double(this->bitmapSize - (2*this->nbPixelBorder)) * width  / height);
	this->imageH = int(height > width ? this->bitmapSize - (2*this->nbPixelBorder) :
		double(this->bitmapSize - (2*this->nbPixelBorder)) * height  / width);

	//facteur entre l'image bitmap et la "map"
	this->factor = double(this->imageW) / width;

	this->imageW += (2*this->nbPixelBorder);
	this->imageH += (2*this->nbPixelBorder);
}

//###############//  
void ns::writeImageInAFile(string filename)
{
	//cre l'image
	QImage image(this->imageW, this->imageH, 32);

	//les couleurs des composants de l'image
	uint backgroundColor = 0xEEEEEE;
	uint labelColor = 0x000000;
	uint labelColorOverlaping = 0xFF0000;
	uint pointColor = 0x0000FF;

	image.fill(backgroundColor); //remplit le fond

	//dessine chaque label
	for(int i=0; i<this->map->getNbPointLabel(); i++)
	{
		//le point
		Point<int> pos = this->conversionToPixmap(this->map->getPointLabel(i)->position);
		
		//dessine le point
		this->drawRect(image, Point<int>(pos.x-1, pos.y-1),
			Point<int>(pos.x+1, pos.y+1), pointColor, false);

		if(!this->solution->isHidden(i))
		{
			//dessine le label
			this->drawRect(image, 
				this->conversionToPixmap(Point<double>
					(this->map->getPointLabel(i)->position.x +
					this->solution->getLabelPlacement(i).x,
					this->map->getPointLabel(i)->position.y +
					this->solution->getLabelPlacement(i).y)),
				this->conversionToPixmap(Point<double>
					(this->map->getPointLabel(i)->position.x +
					this->solution->getLabelPlacement(i).x +
					this->map->getPointLabel(i)->l,
					this->map->getPointLabel(i)->position.y +
					this->solution->getLabelPlacement(i).y +
					this->map->getPointLabel(i)->h)),
				this->solution->labelIsOverlaping(i) &&
					!this->solution->maskingOn() ? labelColorOverlaping : labelColor,
				this->solution->labelIsOverlaping(i) &&
					!this->solution->maskingOn());
		}
	}

	//crit l'image
	image.save(QString(filename), "PNG");

	cout << "Image file \"" << filename << "\" have been written" << endl;
}

//###############//  
void ns::drawRect(QImage& image, const Point<int>& p1,
	const Point<int>& p2, uint color, bool dot) const
{
	//les bords horizontaux
	for(int i=(p1.x<p2.x?p1.x:p2.x)+1; i<(p1.x<p2.x?p2.x:p1.x); i++)
	{
		if(dot && int(i+p1.x/2) % 3 == 0) continue;
		this->drawPixel(image, i, p2.y, color);
		this->drawPixel(image, i, p1.y, color);
	}

	//les bords verticaux
	for(int i=(p1.y<p2.y?p1.y:p2.y); i<=(p1.y<p2.y?p2.y:p1.y); i++)
	{
		if(dot && int(i+p1.y/2) % 3 == 0) continue;
		this->drawPixel(image, p1.x, i, color);
		this->drawPixel(image, p2.x, i, color);
	}
}

//###############//  
void ns::drawPixel(QImage& image, int x, int y, uint color) const
{
	if(x >= 0 && x < image.width() && y >= 0 && y < image.height())
		image.setPixel(x, y, color);
}

//###############//  
Point<int> ns::conversionToPixmap(const Point<double> p) const
{
	return Point<int>(int((p.x - this->minCorner.x) * this->factor) + this->nbPixelBorder,
		int(-(p.y - this->minCorner.y) * this->factor + this->imageH-1) - this->nbPixelBorder);
}
